@@ -103,34 +103,11 @@ static const std::map<std::string, OpenVINOModelTestCaseInfo>& getOpenVINOTestMo
 #if INF_ENGINE_RELEASE >= 2020010000
         // Downloaded using these parameters for Open Model Zoo downloader (2020.1):
         // ./downloader.py -o ${OPENCV_DNN_TEST_DATA_PATH}/omz_intel_models --cache_dir ${OPENCV_DNN_TEST_DATA_PATH}/.omz_cache/ \
-        //     --name person-detection-retail-0013,age-gender-recognition-retail-0013
+        //     --name person-detection-retail-0013
         { "person-detection-retail-0013", {  // IRv10
             "intel/person-detection-retail-0013/FP32/person-detection-retail-0013",
             "intel/person-detection-retail-0013/FP16/person-detection-retail-0013"
         }},
-        { "age-gender-recognition-retail-0013", {
-            "intel/age-gender-recognition-retail-0013/FP16/age-gender-recognition-retail-0013",
-            "intel/age-gender-recognition-retail-0013/FP32/age-gender-recognition-retail-0013"
-        }},
-#endif
-#if INF_ENGINE_RELEASE >= 2021020000
-        // OMZ: 2020.2
-        { "face-detection-0105", {
-            "intel/face-detection-0105/FP32/face-detection-0105",
-            "intel/face-detection-0105/FP16/face-detection-0105"
-        }},
-        { "face-detection-0106", {
-            "intel/face-detection-0106/FP32/face-detection-0106",
-            "intel/face-detection-0106/FP16/face-detection-0106"
-        }},
-#endif
-#if INF_ENGINE_RELEASE >= 2021040000
-        // OMZ: 2021.4
-        { "person-vehicle-bike-detection-2004", {
-            "intel/person-vehicle-bike-detection-2004/FP32/person-vehicle-bike-detection-2004",
-            "intel/person-vehicle-bike-detection-2004/FP16/person-vehicle-bike-detection-2004"
-            //"intel/person-vehicle-bike-detection-2004/FP16-INT8/person-vehicle-bike-detection-2004"
-        }},
 #endif
     };
 
@@ -146,40 +123,13 @@ static const std::vector<std::string> getOpenVINOTestModelsList()
     return result;
 }
 
-inline static std::string getOpenVINOModel(const std::string &modelName, bool isFP16)
-{
-    const std::map<std::string, OpenVINOModelTestCaseInfo>& models = getOpenVINOTestModels();
-    const auto it = models.find(modelName);
-    if (it != models.end())
-    {
-        OpenVINOModelTestCaseInfo modelInfo = it->second;
-        if (isFP16 && modelInfo.modelPathFP16)
-            return std::string(modelInfo.modelPathFP16);
-        else if (!isFP16 && modelInfo.modelPathFP32)
-            return std::string(modelInfo.modelPathFP32);
-    }
-    return std::string();
-}
-
 static inline void genData(const InferenceEngine::TensorDesc& desc, Mat& m, Blob::Ptr& dataPtr)
 {
     const std::vector<size_t>& dims = desc.getDims();
-    if (desc.getPrecision() == InferenceEngine::Precision::FP32)
-    {
-        m.create(std::vector<int>(dims.begin(), dims.end()), CV_32F);
-        randu(m, -1, 1);
-        dataPtr = make_shared_blob<float>(desc, (float*)m.data);
-    }
-    else if (desc.getPrecision() == InferenceEngine::Precision::I32)
-    {
-        m.create(std::vector<int>(dims.begin(), dims.end()), CV_32S);
-        randu(m, -100, 100);
-        dataPtr = make_shared_blob<int>(desc, (int*)m.data);
-    }
-    else
-    {
-        FAIL() << "Unsupported precision: " << desc.getPrecision();
-    }
+    m.create(std::vector<int>(dims.begin(), dims.end()), CV_32F);
+    randu(m, -1, 1);
+
+    dataPtr = make_shared_blob<float>(desc, (float*)m.data);
 }
 
 void runIE(Target target, const std::string& xmlPath, const std::string& binPath,
@@ -285,16 +235,7 @@ void runIE(Target target, const std::string& xmlPath, const std::string& binPath
     BlobMap inputBlobs;
     for (auto& it : net.getInputsInfo())
     {
-        const InferenceEngine::TensorDesc& desc = it.second->getTensorDesc();
-        genData(desc, inputsMap[it.first], inputBlobs[it.first]);
-        if (cvtest::debugLevel > 0)
-        {
-            const std::vector<size_t>& dims = desc.getDims();
-            std::cout << "Input: '" << it.first << "' precison=" << desc.getPrecision() << " dims=" << dims.size() << " [";
-            for (auto d : dims)
-                std::cout << " " << d;
-            std::cout << "]  ocv_mat=" << inputsMap[it.first].size << " of " << typeToString(inputsMap[it.first].type()) << std::endl;
-        }
+        genData(it.second->getTensorDesc(), inputsMap[it.first], inputBlobs[it.first]);
     }
     infRequest.SetInput(inputBlobs);
 
@@ -303,16 +244,7 @@ void runIE(Target target, const std::string& xmlPath, const std::string& binPath
     BlobMap outputBlobs;
     for (auto& it : net.getOutputsInfo())
     {
-        const InferenceEngine::TensorDesc& desc = it.second->getTensorDesc();
-        genData(desc, outputsMap[it.first], outputBlobs[it.first]);
-        if (cvtest::debugLevel > 0)
-        {
-            const std::vector<size_t>& dims = desc.getDims();
-            std::cout << "Output: '" << it.first << "' precison=" << desc.getPrecision() << " dims=" << dims.size() << " [";
-            for (auto d : dims)
-                std::cout << " " << d;
-            std::cout << "]  ocv_mat=" << outputsMap[it.first].size << " of " << typeToString(outputsMap[it.first].type()) << std::endl;
-        }
+        genData(it.second->getTensorDesc(), outputsMap[it.first], outputBlobs[it.first]);
     }
     infRequest.SetOutput(outputBlobs);
 
@@ -333,12 +265,6 @@ void runCV(Backend backendId, Target targetId, const std::string& xmlPath, const
     net.setPreferableTarget(targetId);
 
     std::vector<String> outNames = net.getUnconnectedOutLayersNames();
-    if (cvtest::debugLevel > 0)
-    {
-        std::cout << "OpenCV output names: " << outNames.size() << std::endl;
-        for (auto name : outNames)
-            std::cout << "- " << name << std::endl;
-    }
     std::vector<Mat> outs;
     net.forward(outs, outNames);
 
@@ -362,28 +288,6 @@ TEST_P(DNNTestOpenVINO, models)
     ASSERT_FALSE(backendId != DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && backendId != DNN_BACKEND_INFERENCE_ENGINE_NGRAPH) <<
         "Inference Engine backend is required";
 
-#if INF_ENGINE_VER_MAJOR_GE(2021030000)
-    if (targetId == DNN_TARGET_MYRIAD && (false
-            || modelName == "person-detection-retail-0013"  // ncDeviceOpen:1013 Failed to find booted device after boot
-            || modelName == "age-gender-recognition-retail-0013"  // ncDeviceOpen:1013 Failed to find booted device after boot
-            || modelName == "face-detection-0105"  // get_element_type() must be called on a node with exactly one output
-            || modelName == "face-detection-0106"  // get_element_type() must be called on a node with exactly one output
-            || modelName == "person-vehicle-bike-detection-2004"  // 2021.4+: ncDeviceOpen:1013 Failed to find booted device after boot
-        )
-    )
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-    if (targetId == DNN_TARGET_OPENCL && (false
-            || modelName == "face-detection-0106"  // Operation: 2278 of type ExperimentalDetectronPriorGridGenerator(op::v6) is not supported
-        )
-    )
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_OPENCL, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-    if (targetId == DNN_TARGET_OPENCL_FP16 && (false
-            || modelName == "face-detection-0106"  // Operation: 2278 of type ExperimentalDetectronPriorGridGenerator(op::v6) is not supported
-        )
-    )
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_OPENCL_FP16, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-#endif
-
 #if INF_ENGINE_VER_MAJOR_GE(2020020000)
     if (targetId == DNN_TARGET_MYRIAD && backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
     {
@@ -397,12 +301,20 @@ TEST_P(DNNTestOpenVINO, models)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
 #endif
 
-    ASSERT_EQ(DNN_BACKEND_INFERENCE_ENGINE_NGRAPH, backendId);
+    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API);
+    else if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH);
+    else
+        FAIL() << "Unknown backendId";
 
     bool isFP16 = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD);
 
-    const std::string modelPath = getOpenVINOModel(modelName, isFP16);
-    ASSERT_FALSE(modelPath.empty()) << modelName;
+    const std::map<std::string, OpenVINOModelTestCaseInfo>& models = getOpenVINOTestModels();
+    const auto it = models.find(modelName);
+    ASSERT_TRUE(it != models.end()) << modelName;
+    OpenVINOModelTestCaseInfo modelInfo = it->second;
+    std::string modelPath = isFP16 ? modelInfo.modelPathFP16 : modelInfo.modelPathFP32;
 
     std::string xmlPath = findDataFile(modelPath + ".xml", false);
     std::string binPath = findDataFile(modelPath + ".bin", false);
@@ -413,8 +325,6 @@ TEST_P(DNNTestOpenVINO, models)
     if (targetId == DNN_TARGET_MYRIAD)
         resetMyriadDevice();
     EXPECT_NO_THROW(runIE(targetId, xmlPath, binPath, inputsMap, ieOutputsMap)) << "runIE";
-    if (targetId == DNN_TARGET_MYRIAD)
-        resetMyriadDevice();
     EXPECT_NO_THROW(runCV(backendId, targetId, xmlPath, binPath, inputsMap, cvOutputsMap)) << "runCV";
 
     double eps = 0;
@@ -422,14 +332,6 @@ TEST_P(DNNTestOpenVINO, models)
     if (targetId == DNN_TARGET_CPU && checkHardwareSupport(CV_CPU_AVX_512F))
         eps = 1e-5;
 #endif
-#if INF_ENGINE_VER_MAJOR_GE(2021030000)
-    if (targetId == DNN_TARGET_CPU && modelName == "face-detection-0105")
-        eps = 2e-4;
-#endif
-#if INF_ENGINE_VER_MAJOR_GE(2021040000)
-    if (targetId == DNN_TARGET_CPU && modelName == "person-vehicle-bike-detection-2004")
-        eps = 1e-6;
-#endif
 
     EXPECT_EQ(ieOutputsMap.size(), cvOutputsMap.size());
     for (auto& srcIt : ieOutputsMap)
@@ -449,5 +351,46 @@ INSTANTIATE_TEST_CASE_P(/**/,
     )
 );
 
+typedef TestWithParam<Target> DNNTestHighLevelAPI;
+TEST_P(DNNTestHighLevelAPI, predict)
+{
+    initDLDTDataPath();
+
+    Target target = (dnn::Target)(int)GetParam();
+    bool isFP16 = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD);
+
+    OpenVINOModelTestCaseInfo modelInfo = getOpenVINOTestModels().find("age-gender-recognition-retail-0013")->second;
+
+    std::string modelPath = isFP16 ? modelInfo.modelPathFP16 : modelInfo.modelPathFP32;
+
+    std::string xmlPath = findDataFile(modelPath + ".xml");
+    std::string binPath = findDataFile(modelPath + ".bin");
+
+    Model model(xmlPath, binPath);
+    Mat frame = imread(findDataFile("dnn/googlenet_1.png"));
+    std::vector<Mat> outs;
+    model.setPreferableBackend(DNN_BACKEND_INFERENCE_ENGINE);
+    model.setPreferableTarget(target);
+    model.predict(frame, outs);
+
+    Net net = readNet(xmlPath, binPath);
+    Mat input = blobFromImage(frame, 1.0, Size(62, 62));
+    net.setInput(input);
+    net.setPreferableBackend(DNN_BACKEND_INFERENCE_ENGINE);
+    net.setPreferableTarget(target);
+
+    std::vector<String> outNames = net.getUnconnectedOutLayersNames();
+    std::vector<Mat> refs;
+    net.forward(refs, outNames);
+
+    CV_Assert(refs.size() == outs.size());
+    for (int i = 0; i < refs.size(); ++i)
+        normAssert(outs[i], refs[i]);
+}
+
+INSTANTIATE_TEST_CASE_P(/**/,
+    DNNTestHighLevelAPI, testing::ValuesIn(getAvailableTargets(DNN_BACKEND_INFERENCE_ENGINE))
+);
+
 }}
 #endif  // HAVE_INF_ENGINE
